import RolloverRegion;



import java.applet.Applet;

import java.awt.Rectangle;

import java.awt.Image;

import java.awt.Graphics;

import java.awt.Color;

import java.awt.Font;

import java.awt.FontMetrics;

import java.awt.Event;



public class Rollover extends Applet

	{

	RolloverRegion[] regions;

	int numRegions;



	//values of the rollover regions

	int[] values = { 30, 60, 90, 50 };

	Color[] colors = { Color.yellow, Color.lightGray, Color.cyan, Color.pink };

	//and the popup messages

	String shortLabels[] = { "alfalfa", "corn", "barley", "oats" };



	//to handle clipping

	Rectangle lastPopupRectangle, currentPopupRectangle;



	Image backgroundImage, buffer;

	Graphics bufferGraphics;



	int mouseX, mouseY;



	boolean previousPopup;



	/////////////////////////////////////////////////////////////////////////

	public void init()

		{

		mouseX = mouseY = -1;

		lastPopupRectangle = new Rectangle(-1, -1, -1, -1);

		currentPopupRectangle = new Rectangle(-1, -1, -1, -1);

		previousPopup = false;



		//create the background and buffer images

		backgroundImage = createImage(getSize().width, getSize().height);

		buffer = createImage(getSize().width, getSize().height);

		bufferGraphics = buffer.getGraphics();



		//find the drawing scale

		numRegions = shortLabels.length;

		int maxValue = values[0];

		for (int i = 0; i < numRegions; i++)

			if (maxValue < values[i])

				maxValue = values[i];

		int regionWidth = getSize().width / numRegions;

		double valuePerPixel = maxValue / (getSize().height * 0.8);



		//create the rollover regions and tell each region where it is

		Font font = this.getFont();

		regions = new RolloverRegion[numRegions];

		for (int i = 0; i < numRegions; i++)

			{

			int height = (int) ((double) values[i] / valuePerPixel);

			int width = regionWidth;

			int xPosition = i * regionWidth;

			int yPosition = getSize().height - height;

			regions[i] =

				new RolloverRegion(shortLabels[i], font, colors[i],

					new Rectangle(xPosition, yPosition, width, height), this);

			}



		//draw each region into the background image

		Graphics backgroundGraphics = backgroundImage.getGraphics();

		for (int i = 0; i < numRegions; i++)

			regions[i].paint(backgroundGraphics);

		}





	/////////////////////////////////////////////////////////////////////////

	public boolean mouseMove(Event ev, int x, int y)

		{

		mouseX = x; mouseY = y;

		repaint();



		return true;

		}





	/////////////////////////////////////////////////////////////////////////

	public boolean mouseExit(Event ev, int x, int y)

		{

		mouseX = mouseY = -1;

		repaint();



		return true;

		}





	/////////////////////////////////////////////////////////////////////////

	public void update(Graphics graphicsContext)

		{

		paint(graphicsContext);

		}





	/////////////////////////////////////////////////////////////////////////

	public void paint(Graphics graphicsContext)

		{

		//find out which rollover region, if any, the mouse is in

		int regionNum;

		for (regionNum = 0; regionNum < numRegions; regionNum++)

			if (regions[regionNum].contains(mouseX, mouseY))

				break;



		//if the mouse isn't in any rollover region,

		//just redraw everything and return

		if (regionNum == numRegions)

			{

			previousPopup = false;

			graphicsContext.drawImage(backgroundImage, 0, 0, this);



			return;

			}



		//the mouse is over a rollover region; get its popup image

		Image popup = regions[regionNum].getPopupImage();

		currentPopupRectangle.x = mouseX;

		currentPopupRectangle.y = mouseY;

		currentPopupRectangle.width = regions[regionNum].getPopupWidth();

		currentPopupRectangle.height = regions[regionNum].getPopupHeight();



		//clip the graphicsContext to the current (and perhaps past) popup;

		//if there's a previous sprite position,

		//clip to the union of the current and previous sprite positions

		//else clip to the current rectangle only

		if (previousPopup)

			{

			Rectangle clip = currentPopupRectangle.union(lastPopupRectangle);

			graphicsContext.clipRect(clip.x, clip.y, clip.width, clip.height);

			}

		else

			{

			graphicsContext.clipRect(

				currentPopupRectangle.x, currentPopupRectangle.y,

				currentPopupRectangle.width, currentPopupRectangle.height);

			}



		//draw to the screen via the buffer

		bufferGraphics.drawImage(backgroundImage, 0, 0, this);

		bufferGraphics.drawImage(popup, mouseX, mouseY, this);

		graphicsContext.drawImage(buffer, 0, 0, this);



		//save the current popup rectangle's position for the next paint()

		lastPopupRectangle.x = currentPopupRectangle.x;

		lastPopupRectangle.y = currentPopupRectangle.y;

		lastPopupRectangle.width = currentPopupRectangle.width;

		lastPopupRectangle.height = currentPopupRectangle.height;

		previousPopup = true;

		}

	}

